"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

exports.default = init;

var _uberproto = require("uberproto");

var _uberproto2 = _interopRequireDefault(_uberproto);

var _feathersErrors = require("feathers-errors");

var _feathersErrors2 = _interopRequireDefault(_feathersErrors);

var _feathersQueryFilters = require("feathers-query-filters");

var _feathersQueryFilters2 = _interopRequireDefault(_feathersQueryFilters);

var _promiscuous = require("promiscuous");

var _promiscuous2 = _interopRequireDefault(_promiscuous);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Service = function () {
  function Service(options) {
    _classCallCheck(this, Service);

    if (!options.model) {
      throw new Error("You must provide a thinky model");
    }
    this.Model = options.model;
    this.joinModels = options.joinModels || false;
    this.paginate = options.paginate || {};
  }

  _createClass(Service, [{
    key: "extend",
    value: function extend(obj) {
      return _uberproto2.default.extend(obj, this);
    }
  }, {
    key: "get",
    value: function get(id) {
      if (!id) {
        return _promiscuous2.default.reject(new _feathersErrors2.default.NotFound("The query did not find a document and returned null"));
      }
      return this.Model.get(id).getJoin(this.joinModels).run().catch(function () {
        return _promiscuous2.default.reject(new _feathersErrors2.default.NotFound("No record found for id '" + id + "'"));
      }).then(function (result) {
        return result;
      });
    }
  }, {
    key: "_find",
    value: function _find(params, getFilter) {
      params = params.query || {};
      var query = this.Model.getJoin(this.joinModels);
      //Break our some params.
      // console.log(getFilter.toString());
      if (!getFilter) {
        getFilter = _feathersQueryFilters2.default;
      }
      var filters = getFilter(params);
      //console.log("filters",filters);
      //console.log("params",params);
      if (filters.$select) {
        query = query.pluck(filters.$select);
      }
      if (filters.$limit) {
        query = query.limit(filters.$limit);
      }
      if (filters.$sort) {
        Object.keys(filters.$sort).forEach(function (element) {
          query = query.orderBy(element);
        }, this);
      }
      if (filters.$skip) {
        query = query.skip(filters.$skip);
      }
      if (params.$or) {
        query = query.filter(orMapper(params.$or));
        delete params.$or;
      }
      if (params.$not) {
        query = query.filter(notMapper(params.$not));
        delete params.$not;
      }
      //See if any of the name params have a special result on them
      return query.filter(parseQuery(params)).run().then(function (values) {
        var total = 3;
        var paginator = {
          total: total,
          limit: filters.$limit,
          skip: filters.$skip || 0,
          data: values
        };
        return _promiscuous2.default.resolve(paginator);
      });
    }
  }, {
    key: "find",
    value: function find(params) {
      var _this = this;

      var result = this._find(params || {}, function (query) {
        return (0, _feathersQueryFilters2.default)(query, _this.paginate);
      });
      if (!this.paginate.default) {
        return result.then(function (page) {
          return page.data;
        });
      }
      return result;
    }
  }, {
    key: "create",
    value: function create(data) {
      var _this2 = this;

      if (Array.isArray(data)) {
        return _promiscuous2.default.all(data.map(function (current) {
          return _this2._create(current);
        }));
      }
      return this._create(data);
    }
  }, {
    key: "_create",
    value: function _create(data) {
      return new this.Model(data).saveAll();
    }
  }, {
    key: "_remove",
    value: function _remove(id) {
      var result = {};
      return this.get(id).then(function (found) {
        result = found;
        return found.delete().then(function () {

          return result;
        });
      });
    }
  }, {
    key: "remove",
    value: function remove(id, params) {
      var _this3 = this;

      if (!params) {
        return this._remove(id).then(function (f) {
          return f;
        });
      }

      return this._find(params, function (query) {
        return (0, _feathersQueryFilters2.default)(query, _this3.paginate);
      }).then(function (found) {
        // console.log(params);
        // console.log(found);
        return _promiscuous2.default.all(found.data.map(function (current) {
          return _this3._remove(current.id);
        }));
      });
    }
  }, {
    key: "_patch",
    value: function _patch(id, data) {
      return this.get(id).then(function (found) {
        return found.merge(data).save();
      });
    }
  }, {
    key: "patch",
    value: function patch(id, data, params) {
      var _this4 = this;

      if (id === null) {
        return this._find(params).then(function (page) {
          return _promiscuous2.default.all(page.data.map(function (current) {
            return _this4._patch(current.id, data, params);
          }));
        });
      }
      return this._patch(id, data, params);
    }
  }, {
    key: "_update",
    value: function _update(id, data) {
      return this.get(id).then(function (found) {
        return found.merge(data).save();
      });
    }
  }, {
    key: "update",
    value: function update(id, data, params) {
      var _this5 = this;

      if (id === null) {
        return this._find(params).then(function (page) {
          return _promiscuous2.default.all(page.data.map(function (current) {
            return _this5._patch(current.id, data, params);
          }));
        });
      }
      return this._patch(id, data, params);
    }
  }]);

  return Service;
}();

function init(options) {
  return new Service(options);
}

function orMapper(orArray) {
  return function (item) {
    var firstKey = Object.keys(orArray[0])[0];
    var output = item(firstKey).eq(orArray[0][firstKey]);
    delete orArray[0];
    orArray.forEach(function (orItem) {
      var itemKey = Object.keys(orItem)[0];
      output = output.or(item(itemKey).eq(orItem[itemKey]));
    });
    return output;
  };
}

function notMapper(orArray) {
  return function (item) {
    var firstKey = Object.keys(orArray[0])[0];
    var output = item(firstKey).not(orArray[0][firstKey]);
    delete orArray[0];
    orArray.forEach(function (orItem) {
      var itemKey = Object.keys(orItem)[0];
      output = output.not(item(itemKey).eq(orItem[itemKey]));
    });
    return output;
  };
}

/**
 * Pass in a query object to get a ReQL query
 * Must be run after special query params are removed.
 */
function parseQuery(obj) {
  return function (r) {

    var reQuery;
    var theKeys = Object.keys(obj);
    for (var index = 0; index < theKeys.length; index++) {
      var subQuery;
      // The queryObject's key: 'name'
      var qField = theKeys[index];
      // The queryObject's value: 'Alice'
      var qValue = obj[qField];
      // If the qValue is an object, it will have special params in it.
      if ((typeof qValue === "undefined" ? "undefined" : _typeof(qValue)) === "object") {
        switch (Object.keys(obj[qField])[0]) {
          case "$in":
            subQuery = r._r.expr(qValue.$in).contains(r(qField));
            break;
          case "$nin":
            subQuery = r._r.expr(qValue.$nin).contains(r(qField)).not();
            break;
          case "$lt":
            subQuery = r(qField).lt(obj[qField].$lt);
            break;
          case "$lte":
            subQuery = r(qField).le(obj[qField].$lte);
            break;
          case "$gt":
            subQuery = r(qField).gt(obj[qField].$gt);
            break;
          case "$gte":
            subQuery = r(qField).ge(obj[qField].$gte);
            break;
          case "$ne":
            subQuery = r(qField).ne(obj[qField].$ne);
            break;
          case "$eq":
            subQuery = r(qField).eq(obj[qField].$eq);
            break;
        }
      } else {
        subQuery = r(qField).eq(qValue);
      }

      // At the end of the current set of attributes, determine placement.
      if (index === 0) {
        reQuery = subQuery;
      } else {
        reQuery = reQuery.and(subQuery);
      }
    }

    return reQuery || {};
  };
}

init.Service = Service;
